function Window(id,parentId,width,height,transX,transY,moveable,constrXmin,constrYmin,constrXmax,constrYmax,showContent,placeholderStyle,windowBackground,margin,statusBar,titleText,statusText,closeButton,minimizeButton,maximizeButton,titlebarStyle,statusbarStyle,titleTextStyle,statusTextStyle,buttonBgStyle,buttonFgStyle,titlebarHeight,statusBarHeight,functionToCall) {
    this.id = id;
    this.parentId = parentId;
    this.width = width;
    this.height = height;
    this.transX = transX;
    this.transY = transY;
    this.constrXmin = constrXmin;
    this.constrYmin = constrYmin;
    this.constrXmax = constrXmax;
    this.constrYmax = constrYmax;
    this.showContent = showContent;
    this.placeholderStyle = placeholderStyle;
    this.moveable = moveable;
    this.windowBackground = windowBackground;
    this.margin = margin;
    this.statusBar = statusBar;
    this.titleText = titleText;
    this.statusText = statusText;
    this.closeButton = closeButton;
    this.minimizeButton = minimizeButton;
    this.maximizeButton = maximizeButton;
    this.titlebarStyle = titlebarStyle;
    this.statusbarStyle = statusbarStyle;
    this.titleTextStyle = titleTextStyle;
    this.statusTextStyle = statusTextStyle;
    this.buttonBgStyle = buttonBgStyle;
    this.buttonFgStyle = buttonFgStyle;
    this.titlebarHeight = titlebarHeight;
    this.statusBarHeight = statusBarHeight;
	this.functionToCall = functionToCall;
    this.windowGroup = null; //later a reference to the window group
    this.parentGroup = null; //later a reference to the parent group of the window
    this.windowTitlebarGroup = null; //later a reference to the group containing the title bar elements of the group
    this.titleBar = null; //later a reference to the titleBar rectangle
    this.windowMainGroup = null; //later a reference to the group containing the window background and the content
    this.windowContentGroup = null; //later a reference to the content of the window
    this.shadowRect = null; //later a reference to a shadow rectangle
    this.shadowTitleRect = null; //later a reference to a shadow rectangle of the titlebar
    this.statusTextNode = null; //later a reference to the content of the window
    this.panStatus = 0; //0 means not active, 1 means mousedown and initialized 2 means currently panning
    this.minimized = false; //status to indicate if window is minimized
    this.closed = false; //status to indicate if window is closed
    this.removed = false; //status to indicate if window is removed/available
    this.createWindow();
}

//create a new window
Window.prototype.createWindow = function() {
    this.testParent();
    //main group of the window
    this.windowGroup = document.createElementNS(svgNS,"g");
    this.windowGroup.setAttributeNS(null,"id",this.id);
	this.windowGroup.setAttributeNS(null,"transform","translate("+this.transX+","+this.transY+")");
	//create shadowRect to represent window if showContent variable is false
	this.shadowRect = document.createElementNS(svgNS,"rect");
	this.shadowRect.setAttributeNS(null,"width",this.width);
	this.shadowRect.setAttributeNS(null,"height",this.height);
	this.shadowRect.setAttributeNS(null,"class",this.placeholderStyle);
	this.shadowRect.setAttributeNS(null,"display","none");
	this.windowGroup.appendChild(this.shadowRect);
	//create shadowRect to represent window if showContent variable is false
	this.shadowTitleRect = document.createElementNS(svgNS,"rect");
	this.shadowTitleRect.setAttributeNS(null,"width",this.width);
	this.shadowTitleRect.setAttributeNS(null,"height",this.titlebarHeight);
	this.shadowTitleRect.setAttributeNS(null,"class",this.placeholderStyle);
	this.shadowTitleRect.setAttributeNS(null,"display","none");
	this.windowGroup.appendChild(this.shadowTitleRect);
    //group of the window titlebar
    this.windowTitlebarGroup = document.createElementNS(svgNS,"g");
    //group containing background and content
    this.windowMainGroup = document.createElementNS(svgNS,"g");
    this.windowMainGroup.setAttributeNS(null,"id","windowMainGroup"+this.id);
    //group later containing window content
    this.windowContentGroup = document.createElementNS(svgNS,"g");
    //create backgroundRect
    var backgroundRect = document.createElementNS(svgNS,"rect");
    backgroundRect.setAttributeNS(null,"width",this.width);
    backgroundRect.setAttributeNS(null,"height",this.height);
    backgroundRect.setAttributeNS(null,"class",this.windowBackground);
    this.windowMainGroup.appendChild(backgroundRect);
    this.windowMainGroup.appendChild(this.windowContentGroup);
    this.windowGroup.appendChild(this.windowMainGroup);
    //create titlebar
    this.titlebar = document.createElementNS(svgNS,"rect");
    this.titlebar.setAttributeNS(null,"width",this.width);
    this.titlebar.setAttributeNS(null,"height",this.titlebarHeight);
    this.titlebar.setAttributeNS(null,"class",this.titlebarStyle);
    this.titlebar.setAttributeNS(null,"id","titleBar"+this.id);
    this.titlebar.setAttributeNS(null,"cursor","pointer");
    this.titlebar.addEventListener("click",this,false);
    this.titlebar.addEventListener("mousedown",this,false);
    this.windowTitlebarGroup.appendChild(this.titlebar);
    var titletext = document.createElementNS(svgNS,"text");
    titletext.setAttributeNS(null,"x",this.margin);
    titletext.setAttributeNS(null,"y",this.titlebarHeight - this.margin);
    titletext.setAttributeNS(null,"class",this.titleTextStyle);
    titletext.setAttributeNS(null,"pointer-events","none");
    if (this.titleText.length > 0) {
        this.titleTextNode = document.createTextNode(this.titleText);
    }
    else {
        this.titleTextNode = document.createTextNode(" ");        
    }
    titletext.appendChild(this.titleTextNode);
    this.windowTitlebarGroup.appendChild(titletext);
    //test if defs section exists or create a new one
    var defsSection = document.getElementsByTagName("defs").item(0);
    if (!defsSection) {
    	defsSection = document.createElementNS(svgNS,"defs");
    	document.documentElement.appendChild(defsSection);
    }
    //now create buttons
    var buttonPosition = this.width - this.margin - this.titlebarHeight * 0.5;
    var buttonWidth = this.titlebarHeight - this.margin * 2;
    //create close button
    if (this.closeButton) {
    	//test if id closeButton exists in defs section or create a new closeButton symbol
     	var closeButtonSymbol = document.getElementById("closeButton");
     	if (!closeButtonSymbol) {
    		var closeButtonSymbol = document.createElementNS(svgNS,"symbol");
    		closeButtonSymbol.setAttributeNS(null,"id","closeButton");
    		closeButtonSymbol.setAttributeNS(null,"overflow","visible");
    		//create background rect
    		var buttonRect = document.createElementNS(svgNS,"rect");
    		buttonRect.setAttributeNS(null,"x",(buttonWidth / 2 * -1));
    		buttonRect.setAttributeNS(null,"y",(buttonWidth / 2 * -1));
    		buttonRect.setAttributeNS(null,"width",buttonWidth);
    		buttonRect.setAttributeNS(null,"height",buttonWidth);
    		buttonRect.setAttributeNS(null,"class",this.buttonBgStyle);
    		buttonRect.setAttributeNS(null,"pointer-events","fill");
    		closeButtonSymbol.appendChild(buttonRect);
    		var buttonLine = document.createElementNS(svgNS,"line");
    		buttonLine.setAttributeNS(null,"x1",(buttonWidth / 2 * -1));
    		buttonLine.setAttributeNS(null,"x2",(buttonWidth / 2));
    		buttonLine.setAttributeNS(null,"y1",(buttonWidth / 2 * -1));
    		buttonLine.setAttributeNS(null,"y2",(buttonWidth / 2));
    		buttonLine.setAttributeNS(null,"class",this.buttonFgStyle);
    		buttonLine.setAttributeNS(null,"pointer-events","none");
    		closeButtonSymbol.appendChild(buttonLine);
    		var buttonLine = document.createElementNS(svgNS,"line");
    		buttonLine.setAttributeNS(null,"x1",(buttonWidth / 2));
    		buttonLine.setAttributeNS(null,"x2",(buttonWidth / 2 * -1));
    		buttonLine.setAttributeNS(null,"y1",(buttonWidth / 2 * -1));
    		buttonLine.setAttributeNS(null,"y2",(buttonWidth / 2));
    		buttonLine.setAttributeNS(null,"class",this.buttonFgStyle);
    		buttonLine.setAttributeNS(null,"pointer-events","none");
    		closeButtonSymbol.appendChild(buttonLine);
    		defsSection.appendChild(closeButtonSymbol);
    	}
    	var closeButtonInstance = document.createElementNS(svgNS,"use");
    	closeButtonInstance.setAttributeNS(null,"x",buttonPosition);
    	closeButtonInstance.setAttributeNS(null,"y",this.titlebarHeight * 0.5);
    	closeButtonInstance.setAttributeNS(null,"cursor","pointer");
        closeButtonInstance.setAttributeNS(null,"id","closeButton"+this.id);        
    	closeButtonInstance.setAttributeNS(xlinkNS,"href","#closeButton");
    	closeButtonInstance.addEventListener("click",this,false);
    	this.windowTitlebarGroup.appendChild(closeButtonInstance);
    	buttonPosition -=  this.titlebarHeight;
    }
 
    //create maximize button
    if (this.maximizeButton) {
    	//test if id maximizeButton exists in defs section or create a new maximizeButton symbol
     	var maximizeButtonSymbol = document.getElementById("maximizeButton");
     	if (!maximizeButtonSymbol) {
    		var maximizeButtonSymbol = document.createElementNS(svgNS,"symbol");
    		maximizeButtonSymbol.setAttributeNS(null,"id","maximizeButton");
    		maximizeButtonSymbol.setAttributeNS(null,"overflow","visible");
    		//create background rect
    		var buttonRect = document.createElementNS(svgNS,"rect");
    		buttonRect.setAttributeNS(null,"x",(buttonWidth / 2 * -1));
    		buttonRect.setAttributeNS(null,"y",(buttonWidth / 2 * -1));
    		buttonRect.setAttributeNS(null,"width",buttonWidth);
    		buttonRect.setAttributeNS(null,"height",buttonWidth);
    		buttonRect.setAttributeNS(null,"class",this.buttonFgStyle);
    		buttonRect.setAttributeNS(null,"pointer-events","fill");
    		maximizeButtonSymbol.appendChild(buttonRect);
    		defsSection.appendChild(maximizeButtonSymbol);
    	}
    	var maximizeButtonInstance = document.createElementNS(svgNS,"use");
    	maximizeButtonInstance.setAttributeNS(null,"x",buttonPosition);
    	maximizeButtonInstance.setAttributeNS(null,"y",this.titlebarHeight * 0.5);
    	maximizeButtonInstance.setAttributeNS(null,"cursor","pointer");
        maximizeButtonInstance.setAttributeNS(null,"id","maximizeButton"+this.id);        
    	maximizeButtonInstance.setAttributeNS(xlinkNS,"href","#maximizeButton");
    	maximizeButtonInstance.addEventListener("click",this,false);
    	this.windowTitlebarGroup.appendChild(maximizeButtonInstance);
    	buttonPosition -=  this.titlebarHeight;
    }

    //create minimize button
    if (this.minimizeButton) {
    	//test if id minimizeButton exists in defs section or create a new minimizeButton symbol
     	var minimizeButtonSymbol = document.getElementById("minimizeButton");
     	if (!minimizeButtonSymbol) {
    		var minimizeButtonSymbol = document.createElementNS(svgNS,"symbol");
    		minimizeButtonSymbol.setAttributeNS(null,"id","minimizeButton");
    		minimizeButtonSymbol.setAttributeNS(null,"overflow","visible");
    		//create background rect
    		var buttonRect = document.createElementNS(svgNS,"rect");
    		buttonRect.setAttributeNS(null,"x",(buttonWidth / 2 * -1));
    		buttonRect.setAttributeNS(null,"y",(buttonWidth / 2 * -1));
    		buttonRect.setAttributeNS(null,"width",buttonWidth);
    		buttonRect.setAttributeNS(null,"height",buttonWidth);
    		buttonRect.setAttributeNS(null,"class",this.buttonBgStyle);
    		buttonRect.setAttributeNS(null,"pointer-events","fill");
    		minimizeButtonSymbol.appendChild(buttonRect);
    		//create line
    		var buttonLine = document.createElementNS(svgNS,"line");
    		buttonLine.setAttributeNS(null,"x1",(buttonWidth / 2));
    		buttonLine.setAttributeNS(null,"x2",(buttonWidth / 2 * -1));
    		buttonLine.setAttributeNS(null,"y1",(buttonWidth / 2));
    		buttonLine.setAttributeNS(null,"y2",(buttonWidth / 2));
    		buttonLine.setAttributeNS(null,"class",this.buttonFgStyle);
    		buttonLine.setAttributeNS(null,"pointer-events","none");
    		minimizeButtonSymbol.appendChild(buttonLine);
    		defsSection.appendChild(minimizeButtonSymbol);
    	}
    	var minimizeButtonInstance = document.createElementNS(svgNS,"use");
    	minimizeButtonInstance.setAttributeNS(null,"x",buttonPosition);
    	minimizeButtonInstance.setAttributeNS(null,"y",this.titlebarHeight * 0.5);
    	minimizeButtonInstance.setAttributeNS(null,"cursor","pointer");
        minimizeButtonInstance.setAttributeNS(null,"id","minimizeButton"+this.id);        
    	minimizeButtonInstance.setAttributeNS(xlinkNS,"href","#minimizeButton");
    	minimizeButtonInstance.addEventListener("click",this,false);
    	this.windowTitlebarGroup.appendChild(minimizeButtonInstance);
    	buttonPosition -=  this.titlebarHeight;
    }

	if (this.statusBar) {
        var statusbar = document.createElementNS(svgNS,"rect");
        statusbar.setAttributeNS(null,"y",(this.height - this.statusBarHeight));
        statusbar.setAttributeNS(null,"width",this.width);
        statusbar.setAttributeNS(null,"height",this.statusBarHeight);
        statusbar.setAttributeNS(null,"class",this.statusbarStyle);
        this.windowMainGroup.appendChild(statusbar);
        var statusTextElement = document.createElementNS(svgNS,"text");
        statusTextElement.setAttributeNS(null,"x",this.margin);
        statusTextElement.setAttributeNS(null,"y",this.height - this.margin);
        statusTextElement.setAttributeNS(null,"class",this.statusTextStyle);
        statusTextElement.setAttributeNS(null,"pointer-events","none");
        if (this.statusText.length > 0) {
            this.statusTextNode = document.createTextNode(this.statusText);
        }
        else {
            this.statusTextNode = document.createTextNode(" ");        
        }
        statusTextElement.appendChild(this.statusTextNode);
        this.windowMainGroup.appendChild(statusTextElement);
	}
	
	//append titlebar group to window group
	this.windowGroup.appendChild(this.windowTitlebarGroup);
	
    //finally append group to windows group
    this.parentGroup.appendChild(this.windowGroup);
}

//test if window group exists or create a new group at the end of the file
Window.prototype.testParent = function() {
    //first test if Windows group exists
    if (!document.getElementById(this.parentId)) {
        this.parentGroup = document.createElementNS(svgNS,"g");
        this.parentGroup.setAttributeNS(null,"id",this.parentId);
        document.documentElement.appendChild(this.parentGroup);
   }
   else {
       this.parentGroup = document.getElementById(this.parentId);
   }
}

//central mouse-event handling
Window.prototype.handleEvent = function(evt) {
    if (evt.type == "click") {
    	var elId = evt.currentTarget.getAttributeNS(null,"id");
        if (elId == "closeButton"+this.id) {
            this.close();
        }
        if (elId == "maximizeButton"+this.id) {
            this.maximize();
        }
        if (elId == "minimizeButton"+this.id) {
            this.minimize();
        }
        if (elId == "titleBar"+this.id) {
            if (evt.detail == 2) {
            	if (this.minimized) {
            		this.maximize();
            	}
            	else {
            		this.minimize();
            	}
            }
        }
    }
    if (evt.type == "mousedown") {
    	var elId = evt.target.getAttributeNS(null,"id");
    	if (elId == "titleBar"+this.id) {
    		//put it to the front
    		this.parentGroup.appendChild(this.windowGroup);
    		if (this.moveable) {
    			this.panStatus = 1;
    			//var coords = myMapApp.calcCoord(evt,document.documentElement);
    			var coords = myMapApp.calcCoord(evt,this.parentGroup);
				this.panCoords = coords;
    			document.documentElement.addEventListener("mousemove",this,false);
    			document.documentElement.addEventListener("mouseup",this,false);
    			if (!this.showContent) {
    				this.windowTitlebarGroup.setAttributeNS(null,"display","none");
    				this.windowMainGroup.setAttributeNS(null,"display","none");
    				if (this.minimized) {
    					this.shadowTitleRect.setAttributeNS(null,"display","inherit");    					
    				}
    				else {
    					this.shadowRect.setAttributeNS(null,"display","inherit");
    				}
    			}
    			else {
    				this.titlebar.setAttributeNS(null,"cursor","move");    			
    			}
    			this.windowGroup.setAttributeNS(batikNS,"static","true");
    		}
    	}
    }
    if (evt.type == "mousemove") {
    	if (this.panStatus == 1) {
			//var coords = myMapApp.calcCoord(evt,document.documentElement);
    		var coords = myMapApp.calcCoord(evt,this.parentGroup);
			if (coords.x < this.constrXmin || coords.x > this.constrXmax || coords.y < this.constrYmin || coords.y > this.constrYmax) {
				this.stopDrag();
			}
			else {
				this.transX += coords.x - this.panCoords.x;
				this.transY += coords.y - this.panCoords.y;
				//check constraints
				if (this.transX < this.constrXmin) {
					this.transX = this.constrXmin;
				}
				if (this.transY < this.constrYmin) {
					this.transY = this.constrYmin;
				}
				if ((this.transX + this.width) > (this.constrXmax)) {
					this.transX = this.constrXmax - this.width;
				}
				if (this.minimized) {
					if ((this.transY + this.titlebarHeight) > (this.constrYmax)) {
						this.transY = this.constrYmax - this.titlebarHeight;
					}			
				}
				else {
					if ((this.transY + this.height) > (this.constrYmax)) {
						this.transY = this.constrYmax - this.height;
					}
				}
				this.windowGroup.setAttributeNS(null,"transform","translate("+this.transX+","+this.transY+")");
    			this.panCoords = coords;
				if (typeof(this.functionToCall) == "string") {
					var functionString = this.functionToCall+"('"+this.id+"','moved')";
					window.setTimeout("eval("+functionString+")",300);
				}
				if (typeof(this.functionToCall) == "function") {
					window.setTimeout(this.functionToCall(this.id,"moved"),300);
				}
				if (typeof(this.functionToCall) == "object") {
					window.setTimeout(this.functionToCall.windowStatusChanged(this.id,"moved"),300);	
				}
				if (typeof(this.functionToCall) == undefined) {
					return;	
				}
    		}
    	}
    }
    if (evt.type == "mouseup") {
    	if (this.panStatus == 1) {
    		this.stopDrag();
    	}    
    }
}

//helper method to stop the dragging mode
Window.prototype.stopDrag = function() {
    	this.windowGroup.removeAttributeNS(batikNS,"static");
    	document.documentElement.removeEventListener("mousemove",this,false);
    	document.documentElement.removeEventListener("mouseup",this,false);
    	if (!this.showContent) {
    		this.windowTitlebarGroup.setAttributeNS(null,"display","inherit");
    		if (this.minimized) {
    			this.shadowTitleRect.setAttributeNS(null,"display","none");    					
    		}
    		else {
    			this.shadowRect.setAttributeNS(null,"display","none");
    			this.windowMainGroup.setAttributeNS(null,"display","inherit");
    		}
    	}
    	else {
    		this.titlebar.setAttributeNS(null,"cursor","pointer");    			
    	}
    	this.panStatus = 0;
}

//minimize a window
Window.prototype.minimize = function() {
			this.windowMainGroup.setAttributeNS(null,"display","none");
			this.minimized = true;
			if (typeof(this.functionToCall) == "string") {
				var functionString = this.functionToCall+"('"+this.id+"','minimized')";
				window.setTimeout("eval("+functionString+")",300);
			}
			if (typeof(this.functionToCall) == "function") {
				window.setTimeout(this.functionToCall(this.id,"minimized"),300);
			}
			if (typeof(this.functionToCall) == "object") {
				window.setTimeout(this.functionToCall.windowStatusChanged(this.id,"minimized"),300);	
			}
			if (typeof(this.functionToCall) == undefined) {
				return;	
			}
}

//maximize a window
Window.prototype.maximize = function() {
			this.windowMainGroup.setAttributeNS(null,"display","inherit");
			if ((this.transY + this.height) > (this.constrYmax)) {
				this.transY = this.constrYmax - this.height;
				this.windowGroup.setAttributeNS(null,"transform","translate("+this.transX+","+this.transY+")");			
			}
			this.minimized = false;
			if (typeof(this.functionToCall) == "string") {
				var functionString = this.functionToCall+"('"+this.id+"','maximized')";
				window.setTimeout("eval("+functionString+")",300);
			}
			if (typeof(this.functionToCall) == "function") {
				window.setTimeout(this.functionToCall(this.id,"maximized"),300);
			}
			if (typeof(this.functionToCall) == "object") {
				window.setTimeout(this.functionToCall.windowStatusChanged(this.id,"maximized"),300);	
			}
			if (typeof(this.functionToCall) == undefined) {
				return;	
			}
}

//open a closed window
Window.prototype.close = function() {
		this.windowGroup.setAttributeNS(null,"display","none");
		this.closed = true;
		if (typeof(this.functionToCall) == "string") {
			var functionString = this.functionToCall+"('"+this.id+"','closed')";
			window.setTimeout("eval("+functionString+")",300);
		}
		if (typeof(this.functionToCall) == "function") {
			window.setTimeout(this.functionToCall(this.id,"closed"),300);
		}
		if (typeof(this.functionToCall) == "object") {
			window.setTimeout(this.functionToCall.windowStatusChanged(this.id,"closed"),300);	
		}
		if (typeof(this.functionToCall) == undefined) {
			return;	
		}
}

//close window, after closing the window is still in its previous state and can be re-opened
Window.prototype.open = function() {
			if (!this.removed) {
				this.windowGroup.setAttributeNS(null,"display","inherit");
				this.closed = false;
				if (typeof(this.functionToCall) == "string") {
					var functionString = this.functionToCall+"('"+this.id+"','opened')";
					window.setTimeout("eval("+functionString+")",300);
				}
				if (typeof(this.functionToCall) == "function") {
					window.setTimeout(this.functionToCall(this.id,"opened"),300);
				}
				if (typeof(this.functionToCall) == "object") {
					window.setTimeout(this.functionToCall.windowStatusChanged(this.id,"opened"),300);	
				}
				if (typeof(this.functionToCall) == undefined) {
					return;	
				}
			}
			else {
				alert("window " + this.id + " is already removed");
			}
}

//remove window
Window.prototype.remove = function() {
	if (!this.removed) {
		if (typeof(this.functionToCall) == "string") {
			var functionString = this.functionToCall+"('"+this.id+"','removed')";
			window.setTimeout("eval("+functionString+")",300);
		}
		if (typeof(this.functionToCall) == "function") {
			window.setTimeout(this.functionToCall(this.id,"removed"),300);
		}
		if (typeof(this.functionToCall) == "object") {
			window.setTimeout(this.functionToCall.windowStatusChanged(this.id,"removed"),300);	
		}
		if (typeof(this.functionToCall) == undefined) {
			return;	
		}
		this.windowGroup.parentGroup.removeChild(this.windowGroup);
		this.removed = true;
	}
}

//change content of statusBar
Window.prototype.setStatusText = function(statusText) {
	if (this.statusBar) {
        this.statusText = statusText;
        if (this.statusText.length > 0) {
            this.statusTextNode.nodeValue = this.statusText;
        }
        else {
            this.statusTextNode.nodeValue = " ";        
        }
	
	}
	else {
		alert("there is no statusbar available");
	}
}

//move a window to a certain position (upper left corner)
Window.prototype.moveTo = function(coordx,coordy) {
	this.transX = coordx;
	this.transY = coordy;
	//check constraints
	if (this.transX < this.constrXmin) {
		this.transX = this.constrXmin;
	}
	if (this.transY < this.constrYmin) {
		this.transY = this.constrYmin;
	}
	if ((this.transX + this.width) > (this.constrXmax)) {
		this.transX = this.constrXmax - this.width;
	}
	if (this.minimized) {
		if ((this.transY + this.titlebarHeight) > (this.constrYmax)) {
			this.transY = this.constrYmax - this.titlebarHeight;
		}			
	}
	else {
		if ((this.transY + this.height) > (this.constrYmax)) {
			this.transY = this.constrYmax - this.height;
		}
	}
	this.windowGroup.setAttributeNS(null,"transform","translate("+this.transX+","+this.transY+")");
	if (typeof(this.functionToCall) == "string") {
		var functionString = this.functionToCall+"('"+this.id+"','movedTo')";
		window.setTimeout("eval("+functionString+")",300);
	}
	if (typeof(this.functionToCall) == "function") {
		window.setTimeout(this.functionToCall(this.id,"movedTo"),300);
	}
	if (typeof(this.functionToCall) == "object") {
		window.setTimeout(this.functionToCall.windowStatusChanged(this.id,"movedTo"),300);	
	}
	if (typeof(this.functionToCall) == undefined) {
		return;	
	}
}

//append new content to the window main group
Window.prototype.appendContent = function(node) {
	if (typeof(node) == "string") {
		node = document.getElementById(node);
	}
	this.windowMainGroup.appendChild(node);
}

//remove new content from the window main group
Window.prototype.removeContent = function(node) {
	if (typeof(node) == "string") {
		node = document.getElementById(node);
	}
	this.windowMainGroup.removeChild(node);
}

//remove new content from the window main group
Window.prototype.insertContentBefore = function(node,referenceNode) {
	if (typeof(node) == "string") {
		node = document.getElementById(node);
	}
	if (typeof(referenceNode) == "string") {
		referenceNode = document.getElementById(referenceNode);
	}
	this.windowMainGroup.insertBefore(node,referenceNode);
}
